////////////////////////////////////////////////////////////////////////////////
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// 
// http://www.apache.org/licenses/LICENSE-2.0 
// 
// Unless required by applicable law or agreed to in writing, software 
// distributed under the License is distributed on an "AS IS" BASIS, 
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and 
// limitations under the License
// 
// No warranty of merchantability or fitness of any kind. 
// Use this software at your own risk.
// 
////////////////////////////////////////////////////////////////////////////////
package actionScripts.ui.editor.text
{
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.utils.clearTimeout;
	import flash.utils.setTimeout;

	import flashx.textLayout.conversion.TextConverter;

	import mx.containers.HBox;
	import mx.managers.PopUpManager;

	import spark.components.RichText;

	public class HoverManager
	{
		protected var editor:TextEditor;
		protected var model:TextEditorModel;

		private var tooltip:HBox;
		private var tooltipText:RichText;
		private var lastTooltipText:String;
		private var hoverTimeoutHandle:int = -1;

		public function HoverManager(editor:TextEditor, model:TextEditorModel)
		{
			this.editor = editor;
			this.model = model;

			tooltip = new HBox();
			tooltip.styleName = "toolTip";
			tooltipText = new RichText();
			tooltip.focusEnabled = false;
			tooltip.mouseEnabled = false;
			tooltip.mouseChildren = false;
			tooltip.addElement(tooltipText);
		}

		public function get isActive():Boolean
		{
			return tooltip.isPopUp;
		}

		public function showHover(contents:Vector.<String>, delayMS:int = 350):void
		{
			//if we're still waiting to show the last one, clear it
			if(hoverTimeoutHandle !== -1)
			{
				clearTimeout(hoverTimeoutHandle);
			}
			var contentsCount:int = contents.length;
			if(contentsCount === 0)
			{
				this.closeHover();
				return;
			}
			if(delayMS === 0)
			{
				this.showHoverAfterDelay(contents);
			}
			else
			{
				hoverTimeoutHandle = setTimeout(showHoverAfterDelay, delayMS, contents);
			}
		}

		public function closeHover():void
		{
			if(hoverTimeoutHandle !== -1)
			{
				clearTimeout(hoverTimeoutHandle);
			}
			if(!tooltip.isPopUp)
			{
				return;
			}
			lastTooltipText = null;
			PopUpManager.removePopUp(tooltip);
			editor.removeEventListener(KeyboardEvent.KEY_DOWN, editor_onKeyDown);
			editor.removeEventListener(MouseEvent.ROLL_OUT, editor_onRollOut);
		}

		private function showHoverAfterDelay(contents:Vector.<String>):void
		{
			hoverTimeoutHandle = -1;
			var contentsCount:int = contents.length;
			var hoverText:String = "";
			for(var i:int = 0; i < contentsCount; i++)
			{
				if(i > 0)
				{
					hoverText += "<br>";
				}
				var content:String = contents[i];
				hoverText += content;
			}
			tooltipText.textFlow = TextConverter.importToFlow(hoverText, TextConverter.TEXT_FIELD_HTML_FORMAT);
			if(!tooltip.isPopUp)
			{
				PopUpManager.addPopUp(tooltip, editor, false);
				tooltip.validateNow();
			}
			if(lastTooltipText !== hoverText)
			{
				lastTooltipText = hoverText;
				var tooltipX:Number = tooltip.stage.mouseX;
				var tooltipY:Number = tooltip.stage.mouseY - (tooltip.height + 15);
				var maxTooltipX:Number = tooltip.stage.stageWidth - tooltip.width;
				if(tooltipX > maxTooltipX)
				{
					tooltipX = maxTooltipX;
				}
				if(tooltipY < 0)
				{
					tooltipY = 0;
				}
				tooltip.move(tooltipX, tooltipY);
			}
			editor.addEventListener(KeyboardEvent.KEY_DOWN, editor_onKeyDown);
			editor.addEventListener(MouseEvent.ROLL_OUT, editor_onRollOut);
		}

		private function editor_onRollOut(event:MouseEvent):void
		{
			this.closeHover();
		}

		private function editor_onKeyDown(event:KeyboardEvent):void
		{
			this.closeHover();
		}
	}
}
